@@ -3,30 +3,31 @@
 
 
 '''
-    ###########################################################################
-    ## FILTER 3D COORDINATES                                                 ##
-    ###########################################################################
-    
-    Filter trc 3D coordinates.
+###########################################################################
+## FILTER 3D COORDINATES                                                 ##
+###########################################################################
 
-    Available filters: Butterworth, Butterworth on speed, Gaussian, LOESS, Median
-    Set your parameters in Config.toml
-        
-    INPUTS: 
-    - a trc file
-    - filtering parameters in Config.toml
-    
-    OUTPUT: 
-    - a filtered trc file
+Filter trc 3D coordinates.
+
+Available filters: Butterworth, Butterworth on speed, Gaussian, LOESS, Median, Kalman
+Set your parameters in Config.toml
     
+INPUTS: 
+- a trc file
+- filtering parameters in Config.toml
+
+OUTPUT: 
+- a filtered trc file
 '''
 
 
 ## INIT
 import os
+import glob
 import fnmatch
 import numpy as np
 import pandas as pd
+import cv2
 import matplotlib.pyplot as plt
 import logging
 
@@ -37,13 +38,14 @@ from filterpy.kalman import KalmanFilter, rts_smoother
 from filterpy.common import Q_discrete_white_noise
 
 from Pose2Sim.common import plotWindow
+from Pose2Sim.common import convert_to_c3d
 
 ## AUTHORSHIP INFORMATION
 __author__ = "David Pagnon"
 __copyright__ = "Copyright 2021, Pose2Sim"
 __credits__ = ["David Pagnon"]
 __license__ = "BSD 3-Clause License"
-__version__ = '0.4'
+__version__ = "0.9.4"
 __maintainer__ = "David Pagnon"
 __email__ = "contact@david-pagnon.com"
 __status__ = "Development"
@@ -135,7 +137,7 @@ def kalman_filter(coords, frame_rate, measurement_noise, process_noise, nb_dimen
     return coords_filt
 
 
-def kalman_filter_1d(config, col):
+def kalman_filter_1d(config_dict, frame_rate, col):
     '''
     1D Kalman filter
     Deals with nans
@@ -143,16 +145,15 @@ def kalman_filter_1d(config, col):
     INPUT:
     - col: Pandas dataframe column
     - trustratio: int, ratio process_noise/measurement_noise
-    - framerate: int
+    - frame_rate: int
     - smooth: boolean, True if double pass (recommended), False if single pass (if real-time)
 
     OUTPUT:
     - col_filtered: Filtered pandas dataframe column
     '''
 
-    trustratio = int(config.get('filtering').get('kalman').get('trust_ratio'))
-    smooth = int(config.get('filtering').get('kalman').get('smooth'))
-    framerate = config.get('project').get('frame_rate')
+    trustratio = int(config_dict.get('filtering').get('kalman').get('trust_ratio'))
+    smooth = int(config_dict.get('filtering').get('kalman').get('smooth'))
     measurement_noise = 20
     process_noise = measurement_noise * trustratio
 
@@ -167,12 +168,12 @@ def kalman_filter_1d(config, col):
     
         # Filter each of the selected sequences
         for seq_f in idx_sequences_to_filter:
-            col_filtered[seq_f] = kalman_filter(col_filtered[seq_f], framerate, measurement_noise, process_noise, nb_dimensions=1, nb_derivatives=3, smooth=smooth).flatten()
+            col_filtered[seq_f] = kalman_filter(col_filtered[seq_f], frame_rate, measurement_noise, process_noise, nb_dimensions=1, nb_derivatives=3, smooth=smooth).flatten()
 
     return col_filtered
 
 
-def butterworth_filter_1d(config, col):
+def butterworth_filter_1d(config_dict, frame_rate, col):
     '''
     1D Zero-phase Butterworth filter (dual pass)
     Deals with nans
@@ -181,18 +182,17 @@ def butterworth_filter_1d(config, col):
     - col: numpy array
     - order: int
     - cutoff: int
-    - framerate: int
+    - frame_rate: int
 
     OUTPUT:
     - col_filtered: Filtered pandas dataframe column
     '''
 
-    type = 'low' #config.get('filtering').get('butterworth').get('type')
-    order = int(config.get('filtering').get('butterworth').get('order'))
-    cutoff = int(config.get('filtering').get('butterworth').get('cut_off_frequency'))    
-    framerate = config.get('project').get('frame_rate')
+    type = 'low' #config_dict.get('filtering').get('butterworth').get('type')
+    order = int(config_dict.get('filtering').get('butterworth').get('order'))
+    cutoff = int(config_dict.get('filtering').get('butterworth').get('cut_off_frequency'))    
 
-    b, a = signal.butter(order/2, cutoff/(framerate/2), type, analog = False) 
+    b, a = signal.butter(order/2, cutoff/(frame_rate/2), type, analog = False) 
     padlen = 3 * max(len(a), len(b))
     
     # split into sequences of not nans
@@ -211,7 +211,7 @@ def butterworth_filter_1d(config, col):
     return col_filtered
     
 
-def butterworth_on_speed_filter_1d(config, col):
+def butterworth_on_speed_filter_1d(config_dict, frame_rate, col):
     '''
     1D zero-phase Butterworth filter (dual pass) on derivative
 
@@ -223,12 +223,11 @@ def butterworth_on_speed_filter_1d(config, col):
     - col_filtered: Filtered pandas dataframe column
     '''
 
-    type = 'low' # config.get('filtering').get('butterworth_on_speed').get('type')
-    order = int(config.get('filtering').get('butterworth_on_speed').get('order'))
-    cutoff = int(config.get('filtering').get('butterworth_on_speed').get('cut_off_frequency'))
-    framerate = config.get('project').get('frame_rate')
+    type = 'low' # config_dict.get('filtering').get('butterworth_on_speed').get('type')
+    order = int(config_dict.get('filtering').get('butterworth_on_speed').get('order'))
+    cutoff = int(config_dict.get('filtering').get('butterworth_on_speed').get('cut_off_frequency'))
 
-    b, a = signal.butter(order/2, cutoff/(framerate/2), type, analog = False)
+    b, a = signal.butter(order/2, cutoff/(frame_rate/2), type, analog = False)
     padlen = 3 * max(len(a), len(b))
     
     # derivative
@@ -252,7 +251,7 @@ def butterworth_on_speed_filter_1d(config, col):
     return col_filtered
 
 
-def gaussian_filter_1d(config, col):
+def gaussian_filter_1d(config_dict, frame_rate, col):
     '''
     1D Gaussian filter
 
@@ -264,14 +263,14 @@ def gaussian_filter_1d(config, col):
     - col_filtered: Filtered pandas dataframe column
     '''
 
-    gaussian_filter_sigma_kernel = int(config.get('filtering').get('gaussian').get('sigma_kernel'))
+    gaussian_filter_sigma_kernel = int(config_dict.get('filtering').get('gaussian').get('sigma_kernel'))
 
     col_filtered = gaussian_filter1d(col, gaussian_filter_sigma_kernel)
 
     return col_filtered
     
 
-def loess_filter_1d(config, col):
+def loess_filter_1d(config_dict, frame_rate, col):
     '''
     1D LOWESS filter (Locally Weighted Scatterplot Smoothing)
 
@@ -284,7 +283,7 @@ def loess_filter_1d(config, col):
     - col_filtered: Filtered pandas dataframe column
     '''
 
-    kernel = config.get('filtering').get('LOESS').get('nb_values_used')
+    kernel = config_dict.get('filtering').get('LOESS').get('nb_values_used')
 
     col_filtered = col.copy()
     mask = np.isnan(col_filtered) 
@@ -301,7 +300,7 @@ def loess_filter_1d(config, col):
     return col_filtered
     
 
-def median_filter_1d(config, col):
+def median_filter_1d(config_dict, frame_rate, col):
     '''
     1D median filter
 
@@ -313,14 +312,14 @@ def median_filter_1d(config, col):
     - col_filtered: Filtered pandas dataframe column
     '''
     
-    median_filter_kernel_size = config.get('filtering').get('median').get('kernel_size')
+    median_filter_kernel_size = config_dict.get('filtering').get('median').get('kernel_size')
     
     col_filtered = signal.medfilt(col, kernel_size=median_filter_kernel_size)
 
     return col_filtered
 
 
-def display_figures_fun(Q_unfilt, Q_filt, time_col, keypoints_names):
+def display_figures_fun(Q_unfilt, Q_filt, time_col, keypoints_names, person_id=0):
     '''
     Displays filtered and unfiltered data for comparison
 
@@ -335,6 +334,7 @@ def display_figures_fun(Q_unfilt, Q_filt, time_col, keypoints_names):
     '''
 
     pw = plotWindow()
+    pw.MainWindow.setWindowTitle('Person '+ str(person_id) + ' coordinates')
     for id, keypoint in enumerate(keypoints_names):
         f = plt.figure()
         
@@ -364,13 +364,14 @@ def display_figures_fun(Q_unfilt, Q_filt, time_col, keypoints_names):
     pw.show()
 
 
-def filter1d(col, config, filter_type):
+def filter1d(col, config_dict, filter_type, frame_rate):
     '''
     Choose filter type and filter column
 
     INPUT:
     - col: Pandas dataframe column
     - filter_type: filter type from Config.toml
+    - frame_rate: int
     
     OUTPUT:
     - col_filtered: Filtered pandas dataframe column
@@ -388,12 +389,12 @@ def filter1d(col, config, filter_type):
     filter_fun = filter_mapping[filter_type]
     
     # Filter column
-    col_filtered = filter_fun(config, col)
+    col_filtered = filter_fun(config_dict, frame_rate, col)
 
     return col_filtered
 
 
-def recap_filter3d(config, trc_path):
+def recap_filter3d(config_dict, trc_path):
     '''
     Print a log message giving filtering parameters. Also stored in User/logs.txt.
 
@@ -402,20 +403,21 @@ def recap_filter3d(config, trc_path):
     '''
 
     # Read Config
-    filter_type = config.get('filtering').get('type')
-    kalman_filter_trustratio = int(config.get('filtering').get('kalman').get('trust_ratio'))
-    kalman_filter_smooth = int(config.get('filtering').get('kalman').get('smooth'))
+    filter_type = config_dict.get('filtering').get('type')
+    kalman_filter_trustratio = int(config_dict.get('filtering').get('kalman').get('trust_ratio'))
+    kalman_filter_smooth = int(config_dict.get('filtering').get('kalman').get('smooth'))
     kalman_filter_smooth_str = 'smoother' if kalman_filter_smooth else 'filter'
-    butterworth_filter_type = 'low' # config.get('filtering').get('butterworth').get('type')
-    butterworth_filter_order = int(config.get('filtering').get('butterworth').get('order'))
-    butterworth_filter_cutoff = int(config.get('filtering').get('butterworth').get('cut_off_frequency'))
-    butter_speed_filter_type = 'low' # config.get('filtering').get('butterworth_on_speed').get('type')
-    butter_speed_filter_order = int(config.get('filtering').get('butterworth_on_speed').get('order'))
-    butter_speed_filter_cutoff = int(config.get('filtering').get('butterworth_on_speed').get('cut_off_frequency'))
-    gaussian_filter_sigma_kernel = int(config.get('filtering').get('gaussian').get('sigma_kernel'))
-    loess_filter_nb_values = config.get('filtering').get('LOESS').get('nb_values_used')
-    median_filter_kernel_size = config.get('filtering').get('median').get('kernel_size')
-
+    butterworth_filter_type = 'low' # config_dict.get('filtering').get('butterworth').get('type')
+    butterworth_filter_order = int(config_dict.get('filtering').get('butterworth').get('order'))
+    butterworth_filter_cutoff = int(config_dict.get('filtering').get('butterworth').get('cut_off_frequency'))
+    butter_speed_filter_type = 'low' # config_dict.get('filtering').get('butterworth_on_speed').get('type')
+    butter_speed_filter_order = int(config_dict.get('filtering').get('butterworth_on_speed').get('order'))
+    butter_speed_filter_cutoff = int(config_dict.get('filtering').get('butterworth_on_speed').get('cut_off_frequency'))
+    gaussian_filter_sigma_kernel = int(config_dict.get('filtering').get('gaussian').get('sigma_kernel'))
+    loess_filter_nb_values = config_dict.get('filtering').get('LOESS').get('nb_values_used')
+    median_filter_kernel_size = config_dict.get('filtering').get('median').get('kernel_size')
+    make_c3d = config_dict.get('filtering').get('make_c3d')
+    
     # Recap
     filter_mapping_recap = {
         'kalman': f'--> Filter type: Kalman {kalman_filter_smooth_str}. Measurements trusted {kalman_filter_trustratio} times as much as previous data, assuming a constant acceleration process.', 
@@ -426,10 +428,12 @@ def recap_filter3d(config, trc_path):
         'median': f'--> Filter type: Median. Kernel size: {median_filter_kernel_size}'
     }
     logging.info(filter_mapping_recap[filter_type])
-    logging.info(f'Filtered 3D coordinates are stored at {trc_path}.')
+    logging.info(f'Filtered 3D coordinates are stored at {trc_path}.\n')
+    if make_c3d:
+        logging.info('All filtered trc files have been converted to c3d.')
 
 
-def filter_all(config):
+def filter_all(config_dict):
     '''
     Filter the 3D coordinates of the trc file.
     Displays filtered coordinates for checking.
@@ -442,56 +446,65 @@ def filter_all(config):
     - a filtered trc file
     '''
 
-    # Read config
-    project_dir = config.get('project').get('project_dir')
-    try:
-        pose_tracked_dir = os.path.join(project_dir, 'pose-associated')
-        os.listdir(pose_tracked_dir)
-        pose_dir = pose_tracked_dir
-    except:
-        pose_dir = os.path.join(project_dir, 'pose')
-    frame_range = config.get('project').get('frame_range')
-    seq_name = os.path.basename(os.path.realpath(project_dir))
+    # Read config_dict
+    project_dir = config_dict.get('project').get('project_dir')
     pose3d_dir = os.path.realpath(os.path.join(project_dir, 'pose-3d'))
-    display_figures = config.get('filtering').get('display_figures')
-    filter_type = config.get('filtering').get('type')
+    display_figures = config_dict.get('filtering').get('display_figures')
+    filter_type = config_dict.get('filtering').get('type')
+    make_c3d = config_dict.get('filtering').get('make_c3d')
 
-    # Frames range
-    pose_listdirs_names = next(os.walk(pose_dir))[1]
-    json_dirs_names = [k for k in pose_listdirs_names if 'json' in k]
-    json_files_names = [fnmatch.filter(os.listdir(os.path.join(pose_dir, js_dir)), '*.json') for js_dir in json_dirs_names]
-    f_range = [[0,min([len(j) for j in json_files_names])] if frame_range==[] else frame_range][0]
+    # Get frame_rate
+    video_dir = os.path.join(project_dir, 'videos')
+    vid_img_extension = config_dict['pose']['vid_img_extension']
+    video_files = glob.glob(os.path.join(video_dir, '*'+vid_img_extension))
+    frame_rate = config_dict.get('project').get('frame_rate')
+    if frame_rate == 'auto': 
+        try:
+            cap = cv2.VideoCapture(video_files[0])
+            cap.read()
+            if cap.read()[0] == False:
+                raise
+            frame_rate = round(cap.get(cv2.CAP_PROP_FPS))
+        except:
+            frame_rate = 60
     
     # Trc paths
-    trc_f_in = f'{seq_name}_{f_range[0]}-{f_range[1]}.trc'
-    trc_f_out = f'{seq_name}_filt_{filter_type}_{f_range[0]}-{f_range[1]}.trc'
-    trc_path_in = os.path.join(pose3d_dir, trc_f_in)
-    trc_path_out = os.path.join(pose3d_dir, trc_f_out)
+    trc_path_in = [file for file in glob.glob(os.path.join(pose3d_dir, '*.trc')) if 'filt' not in file]
+    trc_f_out = [f'{os.path.basename(t).split(".")[0]}_filt_{filter_type}.trc' for t in trc_path_in]
+    trc_path_out = [os.path.join(pose3d_dir, t) for t in trc_f_out]
     
-    # Read trc header
-    with open(trc_path_in, 'r') as trc_file:
-        header = [next(trc_file) for line in range(5)]
+    for person_id, (t_in, t_out) in enumerate(zip(trc_path_in, trc_path_out)):
+        # Read trc header
+        with open(t_in, 'r') as trc_file:
+            header = [next(trc_file) for line in range(5)]
 
-    # Read trc coordinates values
-    trc_df = pd.read_csv(trc_path_in, sep="\t", skiprows=4)
-    frames_col, time_col = trc_df.iloc[:,0], trc_df.iloc[:,1]
-    Q_coord = trc_df.drop(trc_df.columns[[0, 1]], axis=1)
+        # Read trc coordinates values
+        trc_df = pd.read_csv(t_in, sep="\t", skiprows=4)
+        frames_col, time_col = trc_df.iloc[:,0], trc_df.iloc[:,1]
+        Q_coord = trc_df.drop(trc_df.columns[[0, 1, -1]], axis=1)
 
-    # Filter coordinates
-    Q_filt = Q_coord.apply(filter1d, axis=0, args = [config, filter_type])
+        # Filter coordinates
+        Q_filt = Q_coord.apply(filter1d, axis=0, args = [config_dict, filter_type, frame_rate])
 
-    # Display figures
-    if display_figures:
-        # Retrieve keypoints
-        keypoints_names = pd.read_csv(trc_path_in, sep="\t", skiprows=3, nrows=0).columns[2::3].to_numpy()
-        display_figures_fun(Q_coord, Q_filt, time_col, keypoints_names)
+        # Display figures
+        if display_figures:
+            # Retrieve keypoints
+            keypoints_names = pd.read_csv(t_in, sep="\t", skiprows=3, nrows=0).columns[2::3][:-1].to_numpy()
+            display_figures_fun(Q_coord, Q_filt, time_col, keypoints_names, person_id)
+
+        # Reconstruct trc file with filtered coordinates
+        with open(t_out, 'w') as trc_o:
+            [trc_o.write(line) for line in header]
+            Q_filt.insert(0, 'Frame#', frames_col)
+            Q_filt.insert(1, 'Time', time_col)
+            # Q_filt = Q_filt.fillna(' ')
+            Q_filt.to_csv(trc_o, sep='\t', index=False, header=None, lineterminator='\n')
+
+        # Save c3d
+        if make_c3d:
+            convert_to_c3d(t_out)
+
+        # Recap
+        recap_filter3d(config_dict, t_out)
 
-    # Reconstruct trc file with filtered coordinates
-    with open(trc_path_out, 'w') as trc_o:
-        [trc_o.write(line) for line in header]
-        Q_filt.insert(0, 'Frame#', frames_col)
-        Q_filt.insert(1, 'Time', time_col)
-        Q_filt.to_csv(trc_o, sep='\t', index=False, header=None, lineterminator='\n')
 
-    # Recap
-    recap_filter3d(config, trc_path_out)
